DEFAULTS = {
    # Install
    'kubernetes_version': None,  # must be provided
    'kubernetes_download_base_url': 'https://dl.k8s.io',
    'kubernetes_install_dir': '/usr/local/kubernetes',
    'kubernetes_bin_dir': '/usr/local/bin',
    'kubernetes_conf_dir': '/etc/kubernetes',

    # Config
    'kubernetes_service_cidr': None,  # must be provided
    'kubernetes_master_url': 'http://127.0.0.1',
}



from pyinfra.api import deploy

from .configure import configure_kubeconfig, configure_kubernetes_component
from .install import install_kubernetes


@deploy('Deploy Kubernetes master')
def deploy_kubernetes_master(
    state, host,
    etcd_nodes,
    kube_apiserver_kwargs=None,
    kube_scheduler_kwargs=None,
    kube_controller_manager_kwargs=None,
):
    # Install server components
    install_kubernetes(
        state, host,
        components=(
            'kube-apiserver', 'kube-scheduler', 'kube-controller-manager',
        ),
    )

    # Configure the API server, passing in our etcd nodes
    configure_kubernetes_component(
        state, host,
        'kube-apiserver',
        etcd_nodes=etcd_nodes,
        service_kwargs=kube_apiserver_kwargs,
    )

    # Setup the kube-scheduler service/config
    configure_kubernetes_component(
        state, host,
        'kube-scheduler',
        service_kwargs=kube_scheduler_kwargs,
    )

    # Setup the kube-controller-manager service/config
    configure_kubernetes_component(
        state, host,
        'kube-controller-manager',
        service_kwargs=kube_controller_manager_kwargs,
    )


@deploy('Deploy Kubernetes node')
def deploy_kubernetes_node(
    state, host,
    master_address,
    kubelet_kwargs=None,
    kube_proxy_kwargs=None,
):
    # Install node components
    install_kubernetes(
        state, host,
        components=(
            'kubelet', 'kube-proxy',
        ),
    )

    # Setup the kubeconfig for kubelet & kube-proxy to use
    configure_kubeconfig(
        state, host,
        master_address,
    )

    # Setup the kubelet service/config
    configure_kubernetes_component(
        state, host,
        'kubelet',
        service_kwargs=kubelet_kwargs,
    )

    # Setup the kube-proxy service/config
    configure_kubernetes_component(
        state, host,
        'kube-proxy',
        service_kwargs=kube_proxy_kwargs,
    )



from pyinfra.api import deploy, DeployError
from pyinfra.modules import files, server

from .defaults import DEFAULTS

SERVER_BINARIES = [
    'kube-apiserver',
    'kube-scheduler',
    'kube-controller-manager',
]


@deploy('Install Kubernetes', data_defaults=DEFAULTS)
def install_kubernetes(state, host, components=None):
    if not isinstance(components, (list, tuple)):
        raise TypeError('Invalid components of Kubernetes specified to download.')

    if not host.data.kubernetes_version:
        raise DeployError(
            'No kubernetes_version set for this host, refusing to install Kubernetes!',
        )

    files.directory(
        state, host,
        {'Ensure the Kubernetes install directory exists'},
        '{{ host.data.kubernetes_install_dir }}//{{ host.data.kubernetes_version }}',
    )

    files.directory(
        state, host,
        {'Ensure the Kubernetes environment directory exists'},
        host.data.kubernetes_conf_dir,
    )

    kube_type = (
        'server' if any(
            b in components for b in SERVER_BINARIES
        ) else 'node'
    )

    host.data.kubernetes_version_name = (
        'kubernetes-{0}-linux-'
        'amd64' if host.fact.arch == 'x86_64' else host.fact.arch
    ).format(kube_type)

    host.data.kubernetes_temp_filename = state.get_temp_filename(
        'kubernetes-{0}'.format(host.data.kubernetes_version),
    )

    download_kubernetes = files.download(
        state, host,
        {'Download Kubernetes'},
        (
            '{{ host.data.kubernetes_download_base_url }}/'
            '{{ host.data.kubernetes_version }}/'
            '{{ host.data.kubernetes_version_name }}.tar.gz'
        ),
        '{{ host.data.kubernetes_temp_filename }}',
    )

    server.shell(
        state, host,
        {'Extract Kubernetes'},
        '''
        tar -xzf {{ host.data.kubernetes_temp_filename }} \
        -C {{ host.data.kubernetes_install_dir }}/{{ host.data.kubernetes_version }} \
        --strip-components 1
        ''',
        when=download_kubernetes.changed,
    )

    for binary in components:
        files.link(
            state, host,
            {'Symlink {0} to {1}'.format(binary, host.data.kubernetes_bin_dir)},
            '{{ host.data.kubernetes_bin_dir }}/%s' % binary,  # link
            '{{ host.data.kubernetes_install_dir }}/{{ host.data.kubernetes_version }}/%s/bin/%s' % (kube_type, binary),
        )



from pyinfra.api import deploy
from pyinfra.modules import files, init

from .defaults import DEFAULTS
from .util import get_template_path


def make_service_kwargs(service_kwargs):
    return '\n'.join(
        '    --{0}={1} \\'.format(key, value)
        for key, value in service_kwargs.items()
    )


@deploy('Configure kubeconfig', data_defaults=DEFAULTS)
def configure_kubeconfig(state, host, master_address, filename='kubeconfig.yml'):
    files.template(
        state, host,
        {'Upload kubeconfig'},
        get_template_path('kubeconfig.yml.j2'),
        '{{ host.data.kubernetes_conf_dir }}/%s' % filename,
        master_address=master_address,
    )


@deploy('Configure Kubernetes', data_defaults=DEFAULTS)
def configure_kubernetes_component(
    state, host, component,
    enable_service=True,
    **template_data
):
    generate_service = files.template(
        state, host,
        {'Upload the {0} systemd unit file'.format(component)},
        get_template_path('{0}.service.j2'.format(component)),
        '/etc/systemd/system/{0}.service'.format(component),
    )

    generate_config = files.template(
        state, host,
        {'Upload the {0} env file'.format(component)},
        get_template_path('{0}.conf.j2'.format(component)),
        '{{ host.data.kubernetes_conf_dir }}/%s' % component,
        make_service_kwargs=make_service_kwargs,
        **template_data
    )

    # Start (/enable) the service
    op_name = 'Ensure {0} service is running'.format(component)
    if enable_service:
        op_name = '{0} and enabled'.format(op_name)

    init.systemd(
        state, host,
        {op_name},
        component,
        enabled=enable_service,
        restarted=generate_config.changed,
        daemon_reload=generate_service.changed,
    )



from setuptools import find_packages, setup


if __name__ == '__main__':
    setup(
        version='0.1',
        name='pyinfra-kubernetes',
        author='EDITED',
        author_email='engineering@edited.com',
        url='https://github.com/EDITD/pyinfra-kubernetes',
        license='MIT',
        description='Install & bootstrap Kubernetes clusters with pyinfra.',
        packages=find_packages(),
        install_requires=('pyinfra>=0.5'),
    )
